@@ -14,9 +14,11 @@ import warnings
 import numpy as np
 import scipy.linalg as la
 
+from ._base import _BaseSolver
+
 
 # Solver classes ==============================================================
-class _BaseSolver:
+class _BaseTikhonovSolver(_BaseSolver):
     """Base solver for regularized linear least-squares problems of the form
 
         sum_{i} min_{x_i} ||Ax_i - b_i||^2 + ||P_i x_i||^2.
@@ -28,7 +30,7 @@ class _BaseSolver:
     # Properties: matrices ----------------------------------------------------
     @property
     def A(self):
-        """(k,d) ndarray : "left-hand side" data matrix."""
+        """(k, d) ndarray: "left-hand side" data matrix."""
         return self.__A
 
     @A.setter
@@ -37,7 +39,7 @@ class _BaseSolver:
 
     @property
     def B(self):
-        """(k,r) ndarray : "right-hand side" matrix B = [ b_1 | ... | b_r ]."""
+        """(k, r) ndarray: "right-hand side" matrix B = [ b_1 | ... | b_r ]."""
         return self.__B
 
     @B.setter
@@ -73,10 +75,9 @@ class _BaseSolver:
 
         Parameters
         ----------
-        A : (k,d) ndarray
+        A : (k, d) ndarray
             The "left-hand side" matrix.
-
-        B : (k,r) ndarray
+        B : (k, r) ndarray
             The "right-hand side" matrix B = [ b_1 | b_2 | ... | b_r ].
         """
         # Extract the dimensions of A (and ensure A is two-dimensional).
@@ -88,7 +89,7 @@ class _BaseSolver:
 
         # Check dimensions of b.
         if B.ndim == 1:
-            B = B.reshape((-1,1))
+            B = B.reshape((-1, 1))
         if B.ndim != 2:
             raise ValueError("`B` must be one- or two-dimensional")
         if B.shape[0] != k:
@@ -103,13 +104,6 @@ class _BaseSolver:
         if not trained:
             raise AttributeError("lstsq solver not trained (call fit())")
 
-    # Main methods ------------------------------------------------------------
-    def fit(*args, **kwargs):
-        raise NotImplementedError("fit() implemented by child classes")
-
-    def predict(*args, **kwargs):
-        raise NotImplementedError("predict() implemented by child classes")
-
     # Post-processing ---------------------------------------------------------
     def cond(self):
         """Calculate the 2-norm condition number of the data matrix A."""
@@ -122,26 +116,26 @@ class _BaseSolver:
 
         Parameters
         ----------
-        X : (d,r) ndarray
+        X : (d, r) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
 
         Returns
         -------
         resids : (r,) ndarray or float (r = 1)
-            Data misfits ||Ax_i - b_i||_2^2, i = 1,...,r.
+            Data misfits ||Ax_i - b_i||_2^2, i = 1, ..., r.
         """
         self._check_is_trained()
         if self.r == 1 and X.ndim == 1:
-            X = X.reshape((-1,1))
-        if X.shape != (self.d,self.r):
+            X = X.reshape((-1, 1))
+        if X.shape != (self.d, self.r):
             raise ValueError(f"X.shape = {X.shape} != "
-                             f"{(self.d,self.r)} = (d,r)")
+                             f"{(self.d, self.r)} = (d, r)")
         resids = np.sum((self.A @ X - self.B)**2, axis=0)
         return resids[0] if self.r == 1 else resids
 
 
-class SolverL2(_BaseSolver):
+class SolverL2(_BaseTikhonovSolver):
     """Solve the l2-norm ordinary least-squares problem with L2 regularization:
 
         sum_{i} min_{x_i} ||Ax_i - b_i||_2^2 + ||λx_i||_2^2,    λ ≥ 0,
@@ -151,24 +145,24 @@ class SolverL2(_BaseSolver):
         min_{X} ||AX - B||_F^2 + ||λX||_F^2,                    λ ≥ 0.
     """
     # Validation --------------------------------------------------------------
-    def _process_regularizer(self, λ):
-        """Validate the regularization hyperparameter and return λ^2."""
-        if not np.isscalar(λ):
-            raise TypeError("regularization hyperparameter λ must be "
-                            "a scalar")
-        if λ < 0:
-            raise ValueError("regularization hyperparameter λ must be "
+    def _process_regularizer(self, regularizer):
+        """Validate the regularization hyperparameter and return
+        regularizer^2."""
+        if not np.isscalar(regularizer):
+            raise TypeError("regularization hyperparameter must be a scalar")
+        if regularizer < 0:
+            raise ValueError("regularization hyperparameter must be "
                              "non-negative")
-        return λ**2
+        return regularizer**2
 
     # Helper methods ----------------------------------------------------------
-    def _Σinv(self, λ):
+    def _inv_svals(self, regularizer):
         """Compute the regularized inverse singular value matrix,
         Σ^* = Σ (Σ^2 + (λ^2)I)^{-1}. Note Σ^* = Σ^{-1} for λ = 0.
         """
-        λ2 = self._process_regularizer(λ)
-        Σ = self._Σ
-        return 1 / Σ if λ2 == 0 else Σ / (Σ**2 + λ2)
+        regularizer2 = self._process_regularizer(regularizer)
+        svals = self._svals
+        return 1/svals if regularizer2 == 0 else svals/(svals**2+regularizer2)
 
     # Main methods ------------------------------------------------------------
     def fit(self, A, B):
@@ -176,57 +170,56 @@ class SolverL2(_BaseSolver):
 
         Parameters
         ----------
-        A : (k,d) ndarray
+        A : (k, d) ndarray
             The "left-hand side" matrix.
-
-        B : (k,r) ndarray
+        B : (k, r) ndarray
             The "right-hand side" matrix B = [ b_1 | b_2 | ... | b_r ].
         """
         self._process_fit_arguments(A, B)
 
         # Compute the SVD of A and save what is needed to solve the problem.
-        U,Σ,Vt = la.svd(self.A, full_matrices=False)
+        U, svals, Vt = la.svd(self.A, full_matrices=False)
         self._V = Vt.T
-        self._Σ = Σ
+        self._svals = svals
         self._UtB = U.T @ self.B
 
         return self
 
-    def predict(self, λ):
+    def predict(self, regularizer):
         """Solve the least-squares problem with the non-negative scalar
         regularization hyperparameter λ.
 
         Parameters
         ----------
-        λ : float ≥ 0
+        regularizer : float ≥ 0
             Scalar regularization hyperparameter.
 
         Returns
         -------
-        X : (d,r) or (d,) ndarray
+        X : (d, r) or (d,) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
             The result is flattened to a one-dimensional array if r = 1.
         """
         self._check_is_trained("_V")
 
-        Σinv = self._Σinv(λ).reshape((-1,1))
-        X = self._V @ (Σinv * self._UtB)        # X = V Σinv U.T B
+        svals_inv = self._inv_svals(regularizer).reshape((-1, 1))
+        X = self._V @ (svals_inv * self._UtB)        # X = V svals_inv U.T B
 
         return np.ravel(X) if self.r == 1 else X
 
     # Post-processing ---------------------------------------------------------
     def cond(self):
         """Calculate the 2-norm condition number of the data matrix A."""
-        self._check_is_trained("_Σ")
-        return abs(self._Σ.max() / self._Σ.min())
+        self._check_is_trained("_svals")
+        return abs(self._svals.max() / self._svals.min())
 
-    def regcond(self, λ):
+    def regcond(self, regularizer):
         """Compute the 2-norm condition number of the regularized data matrix.
 
         Parameters
         ----------
-        λ : float ≥ 0
+        regularizer : float ≥ 0
             Scalar regularization hyperparameter.
 
         Returns
@@ -234,31 +227,30 @@ class SolverL2(_BaseSolver):
         rc : float ≥ 0
             cond([A.T | λI.T].T), computed from filtered singular values of A.
         """
-        self._check_is_trained("_Σ")
-        Σ2 = self._Σ**2 + self._process_regularizer(λ)
-        return np.sqrt(Σ2.max() / Σ2.min())
+        self._check_is_trained("_svals")
+        svals2 = self._svals**2 + self._process_regularizer(regularizer)
+        return np.sqrt(svals2.max() / svals2.min())
 
-    def residual(self, X, λ):
+    def residual(self, X, regularizer):
         """Calculate the residual of the regularized problem for each column of
         B = [ b_1 | ... | b_r ], i.e., ||Ax_i - b_i||_2^2 + ||λx_i||_2^2.
 
         Parameters
         ----------
-        X : (d,r) ndarray
+        X : (d, r) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
-
-        λ : float ≥ 0
+        regularizer : float ≥ 0
             Scalar regularization hyperparameter.
 
         Returns
         -------
         resids : (r,) ndarray or float (r = 1)
-            Residuals ||Ax_i - b_i||_2^2 + ||λx_i||_2^2, i = 1,...,r.
+            Residuals ||Ax_i - b_i||_2^2 + ||λx_i||_2^2, i = 1, ..., r.
         """
         self._check_is_trained()
-        λ2 = self._process_regularizer(λ)
-        return self.misfit(X) + λ2*np.sum(X**2, axis=0)
+        regularizer2 = self._process_regularizer(regularizer)
+        return self.misfit(X) + regularizer2*np.sum(X**2, axis=0)
 
 
 class SolverL2Decoupled(SolverL2):
@@ -268,85 +260,88 @@ class SolverL2Decoupled(SolverL2):
         min_{x_i} ||Ax_i - b_i||_2^2 + ||λ_i x_i||_2^2,    λ_i > 0.
     """
     # Validation --------------------------------------------------------------
-    def _check_λs(self, λs):
-        if len(λs) != self.r:
-            raise ValueError("len(λs) != number of columns of B")
+    def _check_regularizers(self, regularizers):
+        if len(regularizers) != self.r:
+            raise ValueError("len(regularizers) != number of columns of B")
 
     # Main methods ------------------------------------------------------------
-    def predict(self, λs):
-        """Solve the least-squares problem with regularization hyperparameters λs.
+    def predict(self, regularizers):
+        """Solve the least-squares problem with regularization hyperparameters
+        regularizers.
 
         Parameters
         ----------
-        λs : sequence of r floats or (r,) ndarray
+        regularizers : sequence of r floats or (r,) ndarray
             Scalar regularization hyperparameters, one for each column of B.
 
         Returns
         -------
-        X : (d,r) or (d,) ndarray
+        X : (d, r) or (d,) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
             The result is flattened to a one-dimensional array if r = 1.
         """
         self._check_is_trained("_V")
-        self._check_λs(λs)
+        self._check_regularizers(regularizers)
 
         # Allocate space for the solution.
-        X = np.empty((self.d,self.r))
+        X = np.empty((self.d, self.r))
 
         # Solve each independent regularized lstsq problem (iteratively).
-        for j, λ in enumerate(λs):
-            Σinv = self._Σinv(λ)
-            X[:,j] = self._V @ (Σinv * self._UtB[:,j])      # X = V Σinv U.T B
+        for j, regularizer in enumerate(regularizers):
+            svals_inv = self._inv_svals(regularizer)
+            # X = V svals_inv U.T B
+            X[:, j] = self._V @ (svals_inv * self._UtB[:, j])
 
         return np.ravel(X) if self.r == 1 else X
 
     # Post-processing ---------------------------------------------------------
-    def regcond(self, λs):
+    def regcond(self, regularizers):
         """Compute the 2-norm condition number of each regularized data matrix.
 
         Parameters
         ----------
-        λs : sequence of r floats or (r,) ndarray
+        regularizers : sequence of r floats or (r,) ndarray
             Scalar regularization hyperparameters, one for each column of B.
 
         Returns
         -------
         rcs : (r,) ndarray
-            cond([A.T | (λ_i I).T].T), i = 1,...,r, computed from filtered
+            cond([A.T | (λ_i I).T].T), i = 1, ..., r, computed from filtered
             singular values of the data matrix A.
         """
-        self._check_is_trained("_Σ")
-        self._check_λs(λs)
-        λ2s = np.array([self._process_regularizer(λ) for λ in λs])
-        Σ2 = self._Σ**2 + λ2s.reshape((-1,1))
-        return np.sqrt(Σ2.max(axis=1) / Σ2.min(axis=1))
+        self._check_is_trained("_svals")
+        self._check_regularizers(regularizers)
+        regularizer_2s = np.array([
+            self._process_regularizer(lm) for lm in regularizers])
+        svals2 = self._svals**2 + regularizer_2s.reshape((-1, 1))
+        return np.sqrt(svals2.max(axis=1) / svals2.min(axis=1))
 
-    def residual(self, X, λs):
+    def residual(self, X, regularizers):
         """Calculate the residual of the regularized problem for each column of
         B = [ b_1 | ... | b_r ], i.e., ||Ax_i - b_i||_2^2 + ||λ_i x_i||_2^2.
 
         Parameters
         ----------
-        X : (d,r) ndarray
+        X : (d, r) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
-
-        λs : sequence of r floats or (r,) ndarray
+        regularizers : sequence of r floats or (r,) ndarray
             Scalar regularization hyperparameters, one for each column of B.
 
         Returns
         -------
         resids : (r,) ndarray
-            Residuals ||Ax_i - b_i||_2^2 + ||λ_i x_i||_2^2, i = 1,...,r.
+            Residuals ||Ax_i - b_i||_2^2 + ||λ_i x_i||_2^2, i = 1, ..., r.
         """
         self._check_is_trained()
-        self._check_λs(λs)
-        λ2s = np.array([self._process_regularizer(λ) for λ in λs])
-        return self.misfit(X) + λ2s*np.sum(X**2, axis=0)
+        self._check_regularizers(regularizers)
+        regularizer_2s = np.array([
+            self._process_regularizer(lm) for lm in regularizers])
+        return self.misfit(X) + regularizer_2s*np.sum(X**2, axis=0)
 
 
-class SolverTikhonov(_BaseSolver):
+class SolverTikhonov(_BaseTikhonovSolver):
     """Solve the l2-norm ordinary least-squares problem with Tikhonov
     regularization:
 
@@ -370,8 +365,8 @@ class SolverTikhonov(_BaseSolver):
             return np.diag(P)
 
         # Two-dimensional input (the regularization matrix).
-        elif P.shape != (self.d,self.d):
-            raise ValueError("P.shape != (d,d) or (d,) where d = A.shape[1]")
+        elif P.shape != (self.d, self.d):
+            raise ValueError("P.shape != (d, d) or (d,) where d = A.shape[1]")
 
         return P
 
@@ -389,10 +384,9 @@ class SolverTikhonov(_BaseSolver):
 
         Parameters
         ----------
-        A : (k,d) ndarray
+        A : (k, d) ndarray
             The "left-hand side" matrix.
-
-        B : (k,r) ndarray
+        B : (k, r) ndarray
             The "right-hand side" matrix B = [ b_1 | b_2 | ... | b_r ].
         """
         self._process_fit_arguments(A, B)
@@ -403,18 +397,22 @@ class SolverTikhonov(_BaseSolver):
 
         return self
 
-    def predict(self, P):
+    def predict(self, P, trynormal=True):
         """Solve the least-squares problem with regularization matrix P.
 
         Parameters
         ----------
-        P : (d,d) or (d,) ndarray
+        P : (d, d) or (d,) ndarray
             Regularization matrix (or the diagonals of the regularization
             matrix if one-dimensional).
+        trynormal : bool
+            If True, attempt to solve the problem via the normal equations,
+            falling back on a full least-squares solver if the problem is
+            too ill-conditioned. If False, skip the normal equations attempt.
 
         Returns
         -------
-        X : (d,r) or (d,) ndarray
+        X : (d, r) or (d,) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
             The result is flattened to a one-dimensional array if r = 1.
@@ -422,16 +420,19 @@ class SolverTikhonov(_BaseSolver):
         self._check_is_trained("_AtA")
 
         P, lhs = self._lhs(P)
-        with warnings.catch_warnings():
-            warnings.filterwarnings("error", category=la.LinAlgWarning)
+        if trynormal:
             try:
-                # Attempt to solve the problem via the normal equations.
-                X = la.solve(lhs, self._rhs, assume_a="pos")
-            except (la.LinAlgError, la.LinAlgWarning) as e:
+                with warnings.catch_warnings():
+                    warnings.filterwarnings("error", category=la.LinAlgWarning)
+                    # Attempt to solve the problem via the normal equations.
+                    X = la.solve(lhs, self._rhs, assume_a="pos")
+            except (la.LinAlgError, la.LinAlgWarning):
                 # For ill-conditioned normal equations, use la.lstsq().
-                print(f"normal equations solve failed, switching lstsq solver")
-                Bpad = np.vstack((self.B, np.zeros((self.d, self.r))))
-                X = la.lstsq(np.vstack((self.A, P)), Bpad)[0]
+                print("normal equations solve failed, switching lstsq solver")
+                trynormal = False
+        if not trynormal:
+            Bpad = np.vstack((self.B, np.zeros((self.d, self.r))))
+            X = la.lstsq(np.vstack((self.A, P)), Bpad)[0]
 
         return np.ravel(X) if self.r == 1 else X
 
@@ -441,7 +442,7 @@ class SolverTikhonov(_BaseSolver):
 
         Parameters
         ----------
-        P : (d,d) or (d,) ndarray
+        P : (d, d) or (d,) ndarray
             Regularization matrix (or the diagonals of the regularization
             matrix if one-dimensional).
 
@@ -459,18 +460,17 @@ class SolverTikhonov(_BaseSolver):
 
         Parameters
         ----------
-        X : (d,r) ndarray
+        X : (d, r) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
-
-        P : (d,d) or (d,) ndarray
+        P : (d, d) or (d,) ndarray
             Regularization matrix (or the diagonals of the regularization
             matrix if one-dimensional).
 
         Returns
         -------
         resids : (r,) ndarray or float (r = 1)
-            Residuals ||Ax_i - b_i||_2^2 + ||Px_i||_2^2, i = 1,...,r.
+            Residuals ||Ax_i - b_i||_2^2 + ||Px_i||_2^2, i = 1, ..., r.
         """
         self._check_is_trained()
         P = self._process_regularizer(P)
@@ -495,13 +495,13 @@ class SolverTikhonovDecoupled(SolverTikhonov):
 
         Parameters
         ----------
-        Ps : sequence of r (d,d) or (d,) ndarrays
+        Ps : sequence of r (d, d) or (d,) ndarrays
             Regularization matrices (or the diagonals of the regularization
             matrices if one-dimensional), one for each column of B.
 
         Returns
         -------
-        X : (d,r) ndarray
+        X : (d, r) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
         """
@@ -509,7 +509,7 @@ class SolverTikhonovDecoupled(SolverTikhonov):
         self._check_Ps(Ps)
 
         # Allocate space for the solution.
-        X = np.empty((self.d,self.r))
+        X = np.empty((self.d, self.r))
 
         # Solve each independent problem (iteratively for now).
         Bpad = None
@@ -519,12 +519,12 @@ class SolverTikhonovDecoupled(SolverTikhonov):
                 warnings.filterwarnings("error", category=la.LinAlgWarning)
                 try:
                     # Attempt to solve the problem via the normal equations.
-                    X[:,j] = la.solve(lhs, self._rhs[:,j], assume_a="pos")
-                except (la.LinAlgError, la.LinAlgWarning) as e:
+                    X[:, j] = la.solve(lhs, self._rhs[:, j], assume_a="pos")
+                except (la.LinAlgError, la.LinAlgWarning):
                     # For ill-conditioned normal equations, use la.lstsq().
                     if Bpad is None:
                         Bpad = np.vstack((self.B, np.zeros((self.d, self.r))))
-                    X[:,j] = la.lstsq(np.vstack((self.A, P)), Bpad[:,j])[0]
+                    X[:, j] = la.lstsq(np.vstack((self.A, P)), Bpad[:, j])[0]
 
         return X
 
@@ -534,7 +534,7 @@ class SolverTikhonovDecoupled(SolverTikhonov):
 
         Parameters
         ----------
-        Ps : sequence of r (d,d) or (d,) ndarrays
+        Ps : sequence of r (d, d) or (d,) ndarrays
             Regularization matrices (or the diagonals of the regularization
             matrices if one-dimensional), one for each column of B.
 
@@ -554,23 +554,22 @@ class SolverTikhonovDecoupled(SolverTikhonov):
 
         Parameters
         ----------
-        X : (d,r) ndarray
+        X : (d, r) ndarray
             Least-squares solution X = [ x_1 | ... | x_r ]; each column is the
             solution to the subproblem with the corresponding column of B.
-
-        Ps : sequence of r (d,d) or (d,) ndarrays
+        Ps : sequence of r (d, d) or (d,) ndarrays
             Regularization matrices (or the diagonals of the regularization
             matrices if one-dimensional), one for each column of B.
 
         Returns
         -------
         resids : (r,) ndarray
-            Residuals ||Ax_i - b_i||_2^2 + ||P_i x_i||_2^2, i = 1,...,r.
+            Residuals ||Ax_i - b_i||_2^2 + ||P_i x_i||_2^2, i = 1, ..., r.
         """
         self._check_is_trained()
         self._check_Ps(Ps)
         misfit = self.misfit(X)
-        Pxs = np.array([np.sum((P @ X[:,j])**2) for j,P in enumerate(Ps)])
+        Pxs = np.array([np.sum((P @ X[:, j])**2) for j, P in enumerate(Ps)])
         return misfit + Pxs
 
 
@@ -583,21 +582,19 @@ def solver(A, B, P):
 
     Parameters
     ----------
-    A : (k,d) ndarray
+    A : (k, d) ndarray
         The "left-hand side" matrix.
-
-    B : (k,r) ndarray
+    B : (k, r) ndarray
         The "right-hand side" matrix B = [ b_1 | b_2 | ... | b_r ].
-
-    P : float >= 0 or ndarray of shapes (r,), (d,), (d,d), (r,d), or (r,d,d)
-        Tikhonov regularization hyperparameter(s). The regularization matrix in the
-        least-squares problem depends on the format of the argument:
+    P : float >= 0 or ndarray of shapes (r,), (d,), (d, d), (r, d), (r, d, d)
+        Tikhonov regularization hyperparameter(s). The regularization matrix
+        in the least-squares problem depends on the format of the argument:
         * float >= 0: `P`*I, a scaled identity matrix.
         * (d,) ndarray: diag(P), a diagonal matrix.
-        * (d,d) ndarray: the matrix `P`.
+        * (d, d) ndarray: the matrix `P`.
         * sequence of length r : the jth entry in the sequence is the
-            regularization hyperparameter for the jth column of `b`. Only valid if
-            `b` is two-dimensional and has exactly r columns.
+            regularization hyperparameter for the jth column of `b`. Only
+            valid if `b` is two-dimensional and has exactly r columns.
 
     Returns
     -------
@@ -607,7 +604,7 @@ def solver(A, B, P):
     """
     d = A.shape[1]
     if B.ndim == 1:
-        B = B.reshape((-1,1))
+        B = B.reshape((-1, 1))
 
     # P is a scalar: single L2-regularized problem.
     if np.isscalar(P):
@@ -619,15 +616,15 @@ def solver(A, B, P):
 
     # P is a dxd matrix (or a 1D array of length d for diagonal P):
     # single Tikhonov-regularized problem.
-    elif isinstance(P, np.ndarray) and (P.shape in [(d,), (d,d)]):
+    elif isinstance(P, np.ndarray) and (P.shape in [(d,), (d, d)]):
         solver = SolverTikhonov()
 
     # P is a sequence of r matrices: decoupled Tikhonov-regularized problems.
-    elif np.shape(P) in [(B.shape[1],d), (B.shape[1],d,d)]:
+    elif np.shape(P) in [(B.shape[1], d), (B.shape[1], d, d)]:
         solver = SolverTikhonovDecoupled()
 
     else:
-        raise ValueError(f"invalid or misaligned input P")
+        raise ValueError("invalid or misaligned input P")
 
     return solver.fit(A, B)
 
@@ -639,25 +636,23 @@ def solve(A, B, P=0):
 
     Parameters
     ----------
-    A : (k,d) ndarray
+    A : (k, d) ndarray
         The "left-hand side" matrix.
-
-    B : (k,r) ndarray
+    B : (k, r) ndarray
         The "right-hand side" matrix B = [ b_1 | b_2 | ... | b_r ].
-
-    P : float >= 0 or ndarray of shapes (r,), (d,), (d,d), (r,d), or (r,d,d)
-        Tikhonov regularization hyperparameter(s). The regularization matrix in the
-        least-squares problem depends on the format of the argument:
+    P : float >= 0 or ndarray of shapes (r,), (d,), (d, d), (r, d), (r, d, d)
+        Tikhonov regularization hyperparameter(s). The regularization matrix
+        in the least-squares problem depends on the format of the argument:
         * float >= 0: `P`*I, a scaled identity matrix.
         * (d,) ndarray: diag(P), a diagonal matrix.
-        * (d,d) ndarray: the matrix `P`.
+        * (d, d) ndarray: the matrix `P`.
         * sequence of length r : the jth entry in the sequence is the
             regularization hyperparameter for the jth column of `b`. Only
             valid if `b` is two-dimensional and has exactly r columns.
 
     Returns
     -------
-    x : (d,) or (d,r) ndarray
+    x : (d,) or (d, r) ndarray
         Least-squares solution. If `b` is a two-dimensional array, then
         each column is a solution to the regularized least-squares problem
         with the corresponding column of b.
