@@ -1,7 +1,7 @@
 # pre/__init__.py
 """Tools for preprocessing data."""
 
-from ._basis import *
-from ._shift_scale import *
+from .basis import *
+from .transform import *
 from ._reprojection import *
 from ._finite_difference import *
